const path = require("path");
const fs = require("fs-extra");
const store = require("./routes/store");
const low = require("lowdb");
const FileSync = require("lowdb/adapters/FileSync");

exports.plugin = {
  name: "db",
  version: "1.0.0",
  async register(server, options) {
    const filename = path.join(
      require("os").homedir(),
      "westar_cache/store.json"
    );
    const dirname = path.dirname(filename);
    await fs.ensureDir(dirname);

    const adapter = new FileSync(filename);
    const db = low(adapter);
    await db.defaults({ sites: [], widgets: [], maps: [] }).write();
    server.app.db = db;

    await cleanStore();
    console.log("begine: ");
    server.route([
      {
        path: "/site/register",
        method: "POST",
        options: store.registerSite(server)
      },
      {
        path: "/site/getconfig",
        method: "POST",
        options: store.getSiteConfig(server)
      },
      {
        path: "/widget/register",
        method: "POST",
        options: store.registerWidget(server)
      },
      {
        path: "/widget/getconfig",
        method: "POST",
        options: store.getWidgetConfig(server)
      },
      {
        path: "/map/register",
        method: "POST",
        options: store.registerMap(server)
      },
      {
        path: "/map/getconfig",
        method: "POST",
        options: store.getMapConfig(server)
      },
      {
        path: "/list/widgets",
        method: ["GET", "POST"],
        options: store.listWidgets(server)
      },
      {
        path: "/list/sites",
        method: ["GET", "POST"],
        options: store.listSites(server)
      }
    ]);

    async function cleanStore() {
      const widgets = await db.get("widgets").value();
      const sites = await db.get("sites").value();

      const widgetsLength = widgets.length;
      const sitesLength = sites.length;

      const newWidgets = [];
      const newSites = [];

      for (let i = 0; i < widgetsLength; i++) {
        const item = widgets[i];
        const state = await filterItem(item, "widget");
        if (state) {
          newWidgets.push(item);
        }
      }
      for (let i = 0; i < sitesLength; i++) {
        const item = sites[i];
        const state = await filterItem(item, "site");
        if (state) {
          newSites.push(item);
        }
      }

      await db.set("widgets", newWidgets).write();
      await db.set("sites", newSites).write();
      return true;
    }

    async function filterItem(item, type) {
      try {
        const { name, root } = item;
        const configFile = path.join(root, "westar-config.json");
        const exists = fs.pathExistsSync(configFile);
        if (exists) {
          const data = fs.readJsonSync(configFile);
          if (data.name === name) {
            return true;
          }
        }
      } catch (err) {}

      try {
        const config = JSON.parse(item.config);

        if (type === "site") {
          await db.get("maps").remove({ site: config.name }).write();
        } else if (type === "widget") {
          await db.get("maps").remove({ widget: config.name }).write();
        }
        console.log("error: ", config);
      } catch (err) {}

      return false;
    }
  }
};
