const path = require("path");
const fs = require("fs-extra");
module.exports = {
  registerSite(server) {
    const { db } = server.app;
    return {
      async handler(request, h) {
        const { root } = request.payload;
        const file = path.join(root, "westar-config.json");
        const data = fs.readJsonSync(file);
        const sites = db.get("sites");

        const old = sites.find({ root }).value();

        if (old) {
          await sites
            .find({ root })
            .assign({
              name: data.name,
              root,
              config: JSON.stringify(data),
              time: Date.now()
            })
            .write();
        } else {
          await sites
            .unshift({
              name: data.name,
              root,
              config: JSON.stringify(data),
              time: Date.now()
            })
            .write();
        }
        return {
          status: true,
          data
        };
      }
    };
  },
  getSiteConfig(server) {
    const { db } = server.app;
    return {
      async handler(request, h) {
        const { name } = request.payload;
        const data = await db.get("sites").find({ name }).value();
        return {
          status: true,
          data
        };
      }
    };
  },
  registerWidget(server) {
    const { db } = server.app;
    return {
      async handler(request, h) {
        const { root } = request.payload;
        const file = path.join(root, "westar-config.json");
        const data = fs.readJsonSync(file);
        const widgets = db.get("widgets");

        const old = widgets.find({ root }).value();
        if (old) {
          await widgets
            .find({ root })
            .assign({
              name: data.name,
              root,
              config: JSON.stringify(data),
              time: Date.now()
            })
            .write();
        } else {
          await widgets
            .unshift({
              name: data.name,
              root,
              config: JSON.stringify(data),
              time: Date.now()
            })
            .write();
        }

        return {
          status: true,
          data
        };
      }
    };
  },
  getWidgetConfig(server) {
    const { db } = server.app;
    return {
      async handler(request, h) {
        const { name } = request.payload;
        const data = await db.get("widgets").find({ name }).value();
        return {
          status: true,
          data
        };
      }
    };
  },
  registerMap(server) {
    const { db } = server.app;
    return {
      async handler(request, h) {
        const {
          site = "",
          widget = "",
          target = "",
          map = ""
        } = request.payload;
        const maps = db.get("maps");

        const old = maps.find({ site, widget, target }).value();
        if (old) {
          await maps
            .find({ site, widget, target })
            .assign({
              site,
              widget,
              target,
              map: JSON.stringify(map),
              time: Date.now()
            })
            .write();
        } else {
          await maps
            .unshift({
              site,
              widget,
              target,
              map: JSON.stringify(map),
              time: Date.now()
            })
            .write();
        }

        return {
          status: true
        };
      }
    };
  },
  getMapConfig(server) {
    const { db } = server.app;
    return {
      async handler(request, h) {
        const { site = "", widget = "", target = "" } = request.payload;

        const data = await db
          .get("maps")
          .find({ site, widget, target })
          .value();
        return {
          status: true,
          data
        };
      }
    };
  },
  listWidgets(server) {
    const { db } = server.app;
    return {
      async handler(request, h) {
        const data = await db.get("widgets").value();
        return {
          status: true,
          data
        };
      }
    };
  },
  listSites(server) {
    const { db } = server.app;
    return {
      async handler(request, h) {
        const data = await db.get("sites").value();
        return {
          status: true,
          data
        };
      }
    };
  }
};
