var path = require("path");
const Hapi = require("@hapi/hapi");
const Inert = require("@hapi/inert");
const Common = require("westar-common");
module.exports = async (port = 2020) => {
  const server = Hapi.server({
    port,
    router: {
      stripTrailingSlash: true
    },
    routes: {
      cors: {
        origin: ["*"],
        credentials: true
      },
      files: {
        relativeTo: path.join(__dirname, "public")
      }
    }
  });

  const common = new Common(server.info.uri);

  await server.register(Inert);
  await server.register({
    plugin: require("./api"),
    routes: {
      prefix: "/api"
    }
  });
  await server.register({
    plugin: require("./db"),
    routes: {
      prefix: "/db"
    }
  });

  await server.start();
  console.log("主进程已启动：", server.info.port);

  await server.register({
    plugin: require("./ui"),
    routes: {
      prefix: "/ui"
    }
  });
  await server.register({
    plugin: require("./site"),
    options: {
      common
    }
  });
};

process.on("unhandledRejection", (err) => {
  console.log(err);
  process.exit(1);
});
