const chokidar = require("chokidar");
const path = require("path");
const fs = require("fs-extra");
const Site = require("./site");
// const Common = require("westar-common");

// 路径对应的站点
const Sites = {};

module.exports.plugin = {
  name: "site",
  version: "1.0.0",
  async register(server, options) {
    const { common } = options;

    // const common = new Common(host);
    // 自动更新相关配置
    const cwd = process.cwd();

    const watcher = chokidar.watch("**/westar-config.json", {
      cwd,
      ignored: ["**/node_modules/**", "**/dist/**"]
    });

    watcher
      .on("ready", function () {
        console.log("正在启动站点服务，请稍等...");
      })
      .on("add", (uri) => callback(uri, "add"))
      .on("change", (uri) => callback(uri, "change"))

      .on("unlink", (uri) => {
        if (Sites[uri]) {
          for (let target in Sites[uri]) {
            const instance = Sites[uri][target];
            instance && instance.stop();
          }
        }
      });

    async function callback(uri, type) {
      const file = path.join(cwd, uri);
      const dirname = path.dirname(file);
      let config = {};
      let configStatus = true;
      try {
        config = await fs.readJSONSync(file);
      } catch (err) {
        configStatus = false;
      }
      if (!configStatus) {
        return;
      }

      try {
        if (checkConfig(config)) {
          const { name } = config;
          if (config.isWidget) {
            const state = await checkWidgetRepeat({
              widget: name,
              root: dirname
            });
            if (state) {
              await common.registerWidget(dirname);
            } else {
              throw new Error(
                `[${name}] 名称已被使用, 请修改 [${dirname}/westar-config.json] 中的name值！`
              );
            }
          } else {
            const { name, ports } = config;
            const state = await checkSiteRepeat({
              site: name,
              root: dirname
            });
            if (!state) {
              throw new Error(
                `[${name}] 名称已被使用, 请修改 [${dirname}/westar-config.json] 中的name值！`
              );
            }
            if (!Sites[uri]) {
              Sites[uri] = {};
            }
            for (let target in ports) {
              if (!Sites[uri][target]) {
                const site = new Site({
                  root: dirname,
                  host: server.info.uri,
                  target
                });
                await site.init();
                Sites[uri][target] = site;
              }

              const instance = Sites[uri][target];
              const isOpen = !!instance.server.info.started;
              if (!config.close) {
                await common.registerSite(dirname);
                if (!isOpen) {
                  await instance.start();
                }
              } else {
                if (isOpen) {
                  await instance.stop();
                }
              }
            }
          }
        }
      } catch (err) {
        throw err;
      }
    }

    // 检查新增的站点名称是否与已有的冲突
    async function checkSiteRepeat({ site, root }) {
      try {
        const cacheRoot = await common.getSiteRoot(site);
        if (cacheRoot) {
          if (root !== cacheRoot) {
            return false;
          } else {
            return true;
          }
        } else {
          return true;
        }
      } catch (err) {
        return true;
      }
    }

    // 检查新增的站点名称是否与已有的冲突
    async function checkWidgetRepeat({ widget, root }) {
      try {
        const cacheRoot = await common.getWidgetRoot(widget);
        if (cacheRoot) {
          if (root !== cacheRoot) {
            return false;
          } else {
            return true;
          }
        } else {
          return true;
        }
      } catch (err) {
        return true;
      }
    }
  }
};

function checkConfig(json) {
  if (!json.name) {
    return false;
  }
  if (!json.isWidget) {
    if (!json.ports) {
      if (typeof json.ports !== "object") return false;
    }
  }
  return true;
}
