const path = require("path");
const fs = require("fs");
const { addCommonRoute } = require("../util/index");

// const extendTemplate = require("./template");

const Hapi = require("@hapi/hapi");
const Inert = require("@hapi/inert");
const Common = require("westar-common");
module.exports = class Site {
  constructor({ root, host, target = "dev" }) {
    this.root = root;
    this.host = host;
    this.target = target;
  }
  async init(autoStart) {
    const { target = "dev" } = this;
    const common = new Common(this.host);
    const isNormal = (common.isNormal = await common.isNormal());

    const { name: site } = await common.readConfig(this.root);
    await common.registerSite(this.root);
    await common.buildSiteAll({ site, target, force: true });

    const config = (this.config = await common.getSiteConfig(site));
    const siteDist = await common.getSiteDist({ site, target });
    const server = (this.server = Hapi.server({
      port: config.ports[target],
      routes: {
        files: {
          relativeTo: path.join(siteDist, "resource")
        },
        cors: {
          origin: ["*"],
          credentials: true
        }
      }
    }));
    await server.register(Inert);

    if (isNormal) {
      server.app.site = site;
      server.app.target = target;
      server.app.common = common;
      server.app.maps = {
        sites: {},
        widgets: {}
      };
    }
    await addCommonRoute({ server, route: "/{uri*}" });

    if (autoStart) {
      await this.start();
    }
  }
  async start() {
    await this.server.start();
    const { config, target } = this;
    console.log(`${config.ports[target]} 端口已启动`);
  }
  async stop() {
    const { config, target } = this;
    await this.server.stop();
    console.log(`${config.ports[target]} 端口已关闭`);
  }
};

process.on("unhandledRejection", (err) => {
  console.log(err);
  process.exit(1);
});
