const Common = require("westar-common");
const path = require("path");
const fs = require("fs-extra");
const { addCommonRoute } = require("../util/index");
const prefix = "modules";
const { renderPage, renderConfig } = require("./render");
const newDir = path.join(require("os").homedir(), "westar_cache/modules");
const glob = require("glob");

exports.plugin = {
  name: "ui",
  version: "1.0.0",
  async register(server, options) {
    const currentDir = path.join(__dirname, "assets/modules");

    await fs.copy(currentDir, newDir);

    const common = new Common(server.info.uri);
    server.app.common = common;
    server.route([
      {
        path: "/",
        method: "GET",
        handler: async function (request, h) {
          return h.redirect(`/ui/${prefix}/page/index.html`);
        }
      }
    ]);

    await initDatabase({ server });

    await generateConfigFile();

    await initSystemSite({ server });

    async function generateConfigFile() {
      const domain = server.info.uri;
      const json = await renderConfig({ domain });
      fs.outputJSONSync(path.join(newDir, "westar-config.json"), json);
    }
  }
};

async function initDatabase({ server }) {
  const { common } = server.app;
  const cwd = process.cwd();
  const arr = glob.sync("**/westar-config.json", {
    cwd,
    ignored: ["**/node_modules/**", "**/dist/**"]
  });
  const { length } = arr;
  for (let i = 0; i < length; i++) {
    const uri = arr[i];
    const file = path.join(cwd, uri);
    const dirname = path.dirname(file);

    const json = await fs.readJSON(file);
    if (json.isWidget) {
      await common.registerWidget(dirname);
    } else {
      await common.registerSite(dirname);
    }
  }
}

async function initSystemSite({ server }) {
  const root = newDir;
  const { common } = server.app;
  const target = "dev";
  const { name: site } = await common.readConfig(root);

  await common.registerSite(root);
  const widgetLists = await common.listWidgets();
  const { length } = widgetLists;
  for (let i = 0; i < length; i++) {
    const item = widgetLists[i];
    const widget = item.name;
    await common.buildSiteWidget({ site, widget, target, force: true });
  }
  await common.buildSite({ site, target, force: true });

  const siteDist = await common.getSiteDist({ site, target });

  //
  await generateFiles();

  server.app.site = site;
  server.app.target = target;
  server.app.common = common;
  server.app.maps = {
    sites: {},
    widgets: {}
  };

  await addCommonRoute({ server, route: `/${prefix}/{uri*}` });

  async function generateFiles() {
    // const widgetLists = await common.listWidgets();
    const list = widgetLists.map((item) => item.name);

    for (let i = 0; i < list.length; i++) {
      const widget = list[i];
      const { res = {} } = await common.getSiteWidgetMap({
        site,
        widget,
        target
      });
      const html = renderPage({ widget, widgets: list, res });

      const file = path.join(newDir, "page", `${widget}.html`);
      await fs.outputFile(file, html);
      if (i === 0) {
        await fs.outputFile(path.join(newDir, "page/index.html"), html);
      }
    }
  }
}
