const path = require("path");
const fs = require("fs-extra");

const mainListHtml = fs.readFileSync(
  path.join(__dirname, "./assets/render-files/list.html")
);
const htmlListHtml = fs.readFileSync(
  path.join(__dirname, "./assets/render-files/list-html.html")
);
const jsListHtml = fs.readFileSync(
  path.join(__dirname, "./assets/render-files/list-js.html")
);
const cssListHtml = fs.readFileSync(
  path.join(__dirname, "./assets/render-files/list-css.html")
);

function convertHtml(html, params) {
  const names = Object.keys(params);
  const vals = Object.values(params);
  return new Function(...names, "return `" + html + "`;")(...vals);
}

module.exports.renderConfig = function ({ domain }) {
  const json = {
    name: "westar_system_modules_site",
    stable: false,
    close: false,
    ports: {},
    rules: [
      {
        glob: "/static/(**)",
        release: "/resource/westar-private-static/$1"
      }
    ],
    targets: {
      dev: {
        domain: `${domain}/ui/modules`
      }
    }
  };
  return json;
};
module.exports.renderPage = function ({ widget = "", widgets = [], res = {} }) {
  const dropdownHTML = widgets
    .map((item) => {
      return `<a href="${item}.html">${item}</a>`;
    })
    .join('<hr class="layui-bg-gray">');

  const datas = {
    html: [],
    js: [],
    css: []
  };

  Object.keys(res).forEach((id) => {
    const item = res[id];
    const { type } = item;
    item.id = id;
    datas[type].push(item);
  });
  const actives = {
    html: datas.html.length ? ["layui-this", "layui-show"] : ["", ""],
    js:
      !datas.html.length && datas.js.length
        ? ["layui-this", "layui-show"]
        : ["", ""],
    css:
      !datas.html.length && !datas.js.length && datas.css.length
        ? ["layui-this", "layui-show"]
        : ["", ""]
  };

  const htmlList = datas.html
    .map((item) => {
      return convertHtml(htmlListHtml, { item });
    })
    .join("");

  const jsList = datas.js
    .map((item) => {
      return convertHtml(jsListHtml, { item });
    })
    .join("");

  const cssList = datas.css
    .map((item) => {
      return convertHtml(cssListHtml, { item });
    })
    .join("");
  const html = convertHtml(mainListHtml, {
    widget,
    widgets,
    datas,
    actives,
    dropdownHTML,
    htmlList,
    jsList,
    cssList
  });
  return html;
};
