const path = require("path");
const extend = require("extend");
module.exports.addCommonRoute = async function ({ server, route }) {
  server.route({
    method: "GET",
    // path: `/${prefix}/{uri*}`,
    path: route,
    async handler(request, h) {
      const start = Date.now();
      const { site, common, target } = server.app;
      const uri = request.params.uri;
      const { getRenderConfig } = await common.getJsConfig(site);
      const siteDist = await common.getSiteDist({ site, target });
      const siteRoot = await common.getSiteRoot(site);
      const type = path
        .extname(uri)
        .replace(".", "")
        .replace(/[?#][\w\W]*$/gm, "");
      const res = await getRenderConfig({
        target,
        uri,
        type
      });

      const config = extend(
        {
          useType: type === "html" ? "src" : "dist", // 'dist' | 'src'
          distConfig: {
            buildType: 0, // 0 => 不构建 1 => 构建站点 2 => 构建站点和依赖
            force: false // 是否强制构建
          },
          srcConfig: {
            static: false, // 是否作为静态文件原样返回 true | false， 如果需要模板引擎渲染，则设置为true
            buildType: 2, // 0 => 不构建 1 => 构建站点 2 => 构建站点和依赖
            force: false
          }
        },
        res
      );

      if (config.useType === "dist") {
        const { distConfig } = config;
        const { buildType, force } = distConfig;
        if (buildType === 1) {
          await common.buildSite({ site, target, force });
        } else if (buildType === 2) {
          await common.buildSiteAll({ site, target, force });
        }
        return h

          .file(path.join(siteDist, uri), {
            confine: false
          })
          .header("render-time", Date.now() - start)
          .header("render-config", JSON.stringify(config));
      } else {
        const { srcConfig } = config;
        const { static, buildType, force } = srcConfig;
        if (static) {
          if (buildType === 1) {
            await common.buildSite({ site, target, force });
          } else if (buildType === 2) {
            await common.buildSiteAll({ site, target, force });
          }
          return h
            .file(path.join(siteRoot, uri), {
              confine: false
            })
            .header("render-time", Date.now() - start)
            .header("render-config", JSON.stringify(config));
        } else {
          const { errorcode, str } = await common.buildPage({
            site,
            target,
            id: uri,
            force
          });
          if (errorcode) {
            return h
              .response(str)
              .code(errorcode)
              .header("render-time", Date.now() - start)
              .header("render-config", JSON.stringify(config));
          } else {
            return h
              .response(str)
              .header("render-time", Date.now() - start)
              .header("render-config", JSON.stringify(config));
          }
        }
      }
    }
  });
};
